<?xml version="1.0" encoding="UTF-8"?>

<#import "traversal_utils.ftl" as utils>
<#import "macros_common_general.ftl" as com>
<#import "front_page_list_reports.ftl" as fp>

<#-- Initialize common variables -->
<@com.initializeMainVariables/>

<#assign locale = "en" />
<#assign sysDateTime = .now>

<#-- Root entity -->
<#assign rootEntity = entity.root>

<#-- Define the main macro to be used to retrieve output by the traversal approach -->
<#assign mainMacroName = "printIfAttachmentElementNode"/>
<#assign mainMacroArgs = ['docElementNode', 'sectionNode', 'entityDocument', 'level']/>

<#-- Make global the macroCall as well as each macro used in it -->
<#global mainMacroCall="<@" + mainMacroName + " " + mainMacroArgs?join(" ") + "/>"/>
<#global printIfAttachmentElementNode = printIfAttachmentElementNode/>
<#global printAttachmentField = printAttachmentField/>
<#global tableOutput=tableOutput/>

<#-- Variables to store table and index -->
<#assign tableRows></#assign>
<#assign index=0/>

<book version="5.0" xmlns="http://docbook.org/ns/docbook" xmlns:xi="http://www.w3.org/2001/XInclude">

    <#-- Get front page -->
    <@fp.getFrontPage _subject _dossierHeader "List of Attachments"/>

    <#-- Traversal approach: prints one row per claim -->
    <@utils.traversal rootEntity 8/>
    
	<chapter label="1">
		<title role="HEAD-1">List of attachments</title>
     
        <para><emphasis role="bold">Total number of attachments: ${index}</emphasis></para>
        <para role='small'>
            <table border="1">
                <title>Attachments</title>
                <col width="18%"/>
                <col width="18%"/>
                <col width="18%"/>
                <col width="18%"/>
                <col width="10%"/>
                <col width="9%"/>
                <col width="9%"/>
                <tbody>
                    <tr>
                        <#--  <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?>
                            <emphasis role="bold">#</emphasis>
                        </th>  -->
                        <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?>
                            <emphasis role="bold">Section or Entity</emphasis>
                        </th>
                        <th colspan="1"><?dbfo bgcolor="#d6eaf8" ?>
                            <emphasis role="bold">Document name and UUID</emphasis>
                        </th>
                        <th colspan="1"><?dbfo bgcolor="#F9C514" ?>
                            <emphasis role="bold">Attachment field name</emphasis>
                        </th>
                        <th colspan="1"><?dbfo bgcolor="#F9C514" ?>
                            <emphasis role="bold">Attachment file name and md5</emphasis>
                        </th>
                        <th colspan="1"><?dbfo bgcolor="#F9C514" ?>
                            <emphasis role="bold">Attachment remarks</emphasis>
                        </th>
                        <th colspan="1"><?dbfo bgcolor="#F9C514" ?>
                            <emphasis role="bold">Attachment file type</emphasis>
                        </th>
                        <th colspan="1"><?dbfo bgcolor="#F9C514" ?>
                            <emphasis role="bold">Attachment file size</emphasis>
                        </th>
                        
                    </tr>
                    ${tableRows}

                </tbody>
            </table>
        </para>

    </chapter>

</book>


<#----------------------------------------------->
<#-- Macro and function definitions start here -->
<#----------------------------------------------->
<#macro printIfAttachmentElementNode docElementNode sectionNode entityDocument level>
    
    <#-- traverse through the document's meta attachments -->
    <#if level==0>
        <#if docElementNode.attachments?has_content>
            <#assign docAttachments = docElementNode.attachments />
            <#list docAttachments as attachmentKey>
                <#if attachmentKey?has_content>
                    <@printAttachmentField attachmentKey docElementNode sectionNode entityDocument/>
                </#if>
            </#list>
        </#if>
    </#if>

    <#if docElementNode?node_type == "attachment">
        <#-- the string represenetation of the element node represents the attachment key -->
        <#local attachmentKey = docElementNode>
        <@printAttachmentField attachmentKey docElementNode sectionNode entityDocument  />
    </#if>
    <#if docElementNode?node_type == "attachments">
        <#list docElementNode as item>
        <#-- the string represenetation of item represents the attachment key -->
            <#local attachmentKey = item>
            <@printAttachmentField attachmentKey docElementNode sectionNode entityDocument  />
        </#list>
    </#if>
</#macro>

<#macro printAttachmentField attachmentKey fieldNode sectionNode entityDocument >
    
    <#-- Attachments metadata -->
    <#local attachmentMetaData = iuclid.getMetadataForAttachment(attachmentKey)>
    <#local attachmentSize = "">
    <#if attachmentMetaData.size?has_content>
        <#local attachmentSize = attachmentMetaData.size?string["0.####"]>
    </#if>

    <#-- Section -->
    <#local sectionName = "">
    <#if sectionNode?has_content>
        <#if sectionNode.number?has_content>
            <#local sectionName = sectionNode.number + " ">
        </#if>
        <#local sectionName = sectionName + sectionNode.title>
    </#if>

    <#-- Section doc -->
    <#local sectionDoc = fieldNode?root>
    <#local sectionDocType = sectionDoc.documentType>
    <#local sectionDocSubType = sectionDoc.documentSubType>
    <#if !utils.isEntityDocument(sectionDoc)>
        <#local sectionDocType = sectionDocType + "." + sectionDoc.documentSubType>
    </#if>
    
    <#-- Field label -->
    <@iuclid.label for=fieldNode var="fieldLabel"/>

    <#-- Web URL pointing to the document -->
    <#local docUrl=utils.getDocumentUrl(sectionDoc) />

    <#-- Variable storing all table rows -->
    <#assign tableRows>
        ${tableRows}
        <@tableOutput attachmentMetaData attachmentSize sectionDoc docUrl fieldLabel fieldNode sectionName entityDocument true/>
    </#assign>
    <#assign index = index + 1/>

</#macro>

<#macro tableOutput attachmentMetaData attachmentSize sectionDoc docUrl fieldLabel fieldNode sectionName entityDocument includeReferences>

    <tr>

        <#--  <td><#assign index = index + 1/>${index}</td>  -->
        
        <#-- IUCLID Section name and number OR Entity (with inbound references if includeReferences==true) -->
        <#-- NOTE: entity could be added -->
        <td>
            <#if sectionName?has_content>
                <@com.text sectionName />
            <#else>
                ${sectionDoc.documentType}
                <#-- if true, then print inbound references: this slows down a lot the report -->
                <#if includeReferences>
                    <@printInboundReferences sectionDoc/>
                </#if>
            </#if>
        </td>

        <#-- IUCLID document name -->
        <td>
            <ulink url="${docUrl}">
                <@com.text sectionDoc.name />
            </ulink>
            <@com.emptyLine/>
            (<@com.text sectionDoc.documentKey.uuid />)
        </td>

        <#-- Attachment field name -->
        <td>
            <@com.text fieldLabel/>
        </td>
        
        
        <#-- Attachment file name -->
        <td>
            <@com.text attachmentMetaData.filename />

            <#if attachmentMetaData.md5?has_content>
                <@com.emptyLine/>
                <para>(${attachmentMetaData.md5})</para>
            </#if>

        </td>

        <#-- Attachment remarks -->
        <td>
            <@com.text attachmentMetaData.remarks/>
        </td>

        <#-- Attachment media type  -->
        <td>
            <#if attachmentMetaData.mediaType?has_content>
                <para>${attachmentMetaData.mediaType}</para>
            </#if>
        </td>

        <td>
            ${attachmentSize}  
        </td>

       
    </tr>
</#macro>

<#-- NOTE: this function is currently not used -->
<#function getFirstHistoryOnlyOfAttachment attachmentMetaData>

    <#local historyList = iuclid.getHistoryForAttachmentKey(attachmentMetaData.documentKey) />

    <#if !(attachmentMetaData)?has_content>
        <#return [] />
    </#if>

    <#if !(historyList)?has_content>
        <#return [] />
    </#if>

    <#if historyList?has_content>
        <#list historyList as history>
            <#if !(history)?has_content>
                <#return [] />
            <#else/>
                <#return historyList[0] />
            </#if>
        </#list>
    </#if>
</#function>

<#macro printInboundReferences doc>
    <#local isFirst=true/>
    <#local inboundRefs=com.inboundReferences(doc.documentKey)/>
    <#list inboundRefs as inboundRefKey>
        <#if inboundRefKey?has_content >
            <#local inboundRef = iuclid.getDocumentForKey(inboundRefKey) />
            <#if inboundRef?has_content>
                <#if isFirst>
                    <para>referenced in:</para>
                    <#local isFirst=false/>
                </#if>
                <para>
                    <#local inboundRefUrl=utils.getDocumentUrl(inboundRef)/>
                    <ulink url="${inboundRefUrl}">
                        <@com.text inboundRef.name />
                    </ulink>
                    
                </para>
            </#if>
        </#if>
    </#list>
</#macro>